using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.ServiceModel;
using RestCake.Util;


namespace RestCake.Metadata
{
	public class AssemblyMetadata
	{
		private readonly Assembly m_assembly;
		public List<ServiceMetadata> Services { get; private set; }

		public string AssemblyName
		{
			get { return m_assembly.GetName().Name; }
		}


		public AssemblyMetadata(string assmeblyToparse)
			// Using LoadFrom() instead of Load() is very important!  LoadFrom() will resolve dependencies.  Load() will not.
			// This is important, since the likely have dependencies on a data layer (DTO classes in a diff assembly, etc)
			// That will cause errors unless you use LoadFrom(), which will resolve those dependencies.
			: this(Assembly.LoadFrom(Path.GetFullPath(assmeblyToparse)))
		{ }

		public AssemblyMetadata(Assembly assembly)
		{
			m_assembly = assembly;
			Services = new List<ServiceMetadata>();
			populateServices();
		}

		private void populateServices()
		{
			// WCF and RestCake services
			IEnumerable<Type> serviceTypes = ReflectionHelper.GetTypesWithAttribute(m_assembly, typeof (ServiceContractAttribute)) // WCF
				.Concat(ReflectionHelper.GetTypesWithAttribute(m_assembly, typeof (RestServiceAttribute))); // RestCake

			foreach (Type serviceType in serviceTypes)
				Services.Add(new ServiceMetadata(serviceType));
		}


	}
}
